/*
 * File:    armour.c
 * Author:  Holger Klawitter
 */

static armour_globals_t *
InitArmourGlobals(void)
{
  armour_globals_t *ag;
  
  if ((ag = MemPtrNew(sizeof(armour_globals_t)))) {
    ag->valid_bits = 0;
    ag->a = 0;
    ag->higapped = false;
  }
  return ag;
}


static void
FreeArmourGlobals(armour_globals_t *ag)
{
  if (ag)
    MemPtrFree(ag);
}


static void
packChar(UInt8 ch, armour_globals_t *ag)
{
  ag->a = ag->a << 8 | ch;
  ag->valid_bits += 8;
  while (ag->valid_bits >= 7) {
    ch = '!' + (ag->a >> (ag->valid_bits-7) & 0x7f);
    if (ch >= 0x80) ch += '!';
		if( ch == 0xad ) ch = 0xc2; /* extra bugfix fr outlook */
    *(ag->destPtr)++ = ch;
    ag->valid_bits -= 7;
    ag->a &= (1 << ag->valid_bits) - 1;
  }
}


static void
unpackChar(UInt8 ch, armour_globals_t *ag)
{
  if (ch<'!') return;
	if( ch == 0xc2 ) ch = 0xad; /* extra bugfix fr outlook */
  if (ch>=0x80 && ag->higapped) ch -= '!';
  if (ch>'!'+128) return; /* ignore non armour characters */
  ag->a = (ag->a << 7) + ch - '!';
  ag->valid_bits += 7;
  if (ag->valid_bits >= 8) {
    *(ag->destPtr)++ = (ch = (ag->a >> (ag->valid_bits-8) & 0xff));
    ag->valid_bits -= 8;
    ag->a &= (1<< ag->valid_bits) - 1;
  }
}


/*
 * A byte sequence is being interpreted as a stream of bits.
 * The bits will be packed into blocks of 6 bits for each
 * character. Therefore the result string must be 8/6 times
 * as large as the source bit string.
 */
static Char *
armour(UInt8 *src, Int16 n, armour_globals_t *ag)
{
  Char *aText;
  Int16 i;
  
  ag->valid_bits = 0;
  ag->a = 0;
  ag->destPtr = (aText = (Char*)MemPtrNew(sizeof(Char)*((n+6)*8/7)));

  for (i=0; i<n; i++)
    packChar(src[i], ag);

  /* There could be some bits still in the stream. Pushing one zero
   * into will surely emit them.
   */
  if (ag->valid_bits)
    packChar(0, ag);
  *(ag->destPtr) = 0;
  return aText;
}


/*
 * Reverses the process of packChars
 */
static UInt8 *
deArmour(Char* src, Int16* len, armour_globals_t *ag)
{
  UInt8 *sText;
  Int16 i;
  Int16 n = StrLen(src);
  UInt8 ch;

  // Determine armouring method
  ag->higapped = true;
  for (i=0; i<n; i++) {
    ch = src[i];
    if (ch > 0x80) {
      ag->higapped = (ch >= 0xA0);
      break;
    }
  }

  ag->valid_bits = 0;
  ag->a = 0;
  *len = (((n*7/8)+7) / 8) * 8;  
  ag->destPtr = (sText = (UInt8*)MemPtrNew(sizeof(Char)*(*len)));

  for (i=0; i<n; i++)
    unpackChar(src[i], ag);
  /* There might be odd bits in a, but this does not matter as packing already
   * has produced enough characters. If the user selected a sequence of wrong
   * size, she might therefore get a truncated result.
   */
  return sText;
}

// EOF
