/* cipher.c */
#include <PalmOS.h>
#define VoidPtr void*

//#pragma segment 1000

#include "armour.h"
#include "crypt.h"
#include "cipher.h"

void
start()
{
  UInt32 feature = 0;

  // Check and set lock
  if (!FtrGet(CipherDACreatorID, 0, &feature))
    return;
  FtrSet(CipherDACreatorID, 0, feature);

  StartApplication();
  EventLoop();
  StopApplication();

  // Release lock
  FtrUnregister(CipherDACreatorID, 0);
}


#include "armour.c"
#include "crypt.c"


static cipher_globals_t *
InitCipherGlobals(void)
{
  cipher_globals_t *mg;

  if ((mg = MemPtrNew(sizeof(cipher_globals_t))))
    mg->theKey = NULL;
  return mg;
}

static void
FreeCipherGlobals(cipher_globals_t *mg)
{
  if (mg) {
    if (mg->theKey) {
      MemSet(mg->theKey, StrLen(mg->theKey), 0);
      MemPtrFree(mg->theKey);
    }
    MemPtrFree(mg);
  }
}


static VoidPtr
GetObjectPtr(UInt16 objectId)
{
  FormPtr frm;
  frm = FrmGetActiveForm();
  return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectId));
}

#if 0
static Int16
GetObjectIdx(UInt16 objectId)
{
  FormPtr frm = FrmGetActiveForm();
  return FrmGetObjectIndex(frm, objectId);
}
#endif

static void
ComputeKey(cipher_globals_t *mg)
{
  FieldPtr fld = GetObjectPtr(KeyField);
  MemHandle handle = FldGetTextHandle(fld);
  Char* text;
  Int16 len;

  if (!FldDirty(fld) && CtlGetValue(GetObjectPtr(KeyCheck)))
    return; /* keeps old key if checked and no new key present */

  if (!handle)
    return; /* no key */

  if (mg->theKey) {
    MemSet(mg->theKey, StrLen(mg->theKey), 0);
    MemPtrFree(mg->theKey);
  }

  text = MemHandleLock(handle);
  len = StrLen(text);
  mg->theKey = (Char*)MemPtrNew(sizeof(Char)*(len+1));
  StrCopy(mg->theKey, text);
  MemHandleUnlock(handle);
}


static void
CleanKey()
{
  FieldPtr fld = GetObjectPtr(KeyField);
  Char *str = FldGetTextPtr(fld);
  if (str) {
    FldDelete(fld, 0, StrLen(str));
    FldSetDirty(fld, false);
  }
  CtlSetValue(GetObjectPtr(KeyCheck), true);
}


static void
ReplaceText(FieldPtr fld, Char *text)
{
  MemHandle oldHandle = FldGetTextHandle(fld);
  MemHandle newHandle = MemHandleNew(sizeof(Char)*4096);
  Char* newHText = MemHandleLock(newHandle);
  Char* oldText;

  MemMove(newHText, text, StrLen(text)+1);
  MemHandleUnlock(newHandle);

  oldText = MemHandleLock(oldHandle);
  MemSet(oldText, StrLen(oldText), 0);
  MemHandleUnlock(oldHandle);

  FldSetTextHandle(fld, newHandle);
  MemHandleFree(oldHandle);
  FldDrawField(fld);
}


static void
Cipher(Boolean weak, cipher_globals_t *mg, crypt_globals_t *crg, armour_globals_t *ag)
{
  FieldPtr fld = GetObjectPtr(TextField);
  MemHandle handle = FldGetTextHandle(fld);
  Char* text, *cText;

  if (!handle) return; /* nothing to do */
  ComputeKey(mg);
  text = MemHandleLock(handle);
  if (StrLen(text)>=MAX_TEXT_LEN) {
    FrmAlert(TooLongError);
    MemHandleUnlock(handle);
    return;
  }
  cText = encrypt(mg->theKey, text, weak, crg, ag);
  MemHandleUnlock(handle);
  if (cText) {
    ReplaceText(fld, cText);
    MemSet(cText, StrLen(cText), 0);
    MemPtrFree(cText);
  }
  CleanKey();
  CtlSetValue(GetObjectPtr(CopyCheck), true);
}


static void
Decipher(Boolean weak, cipher_globals_t *mg, crypt_globals_t *crg, armour_globals_t *ag)
{
  FieldPtr fld = GetObjectPtr(TextField);
  MemHandle handle = FldGetTextHandle(fld);
  Char *text, *oText;

  if (!handle) return; /* nothing to do */
  ComputeKey(mg);
  text = MemHandleLock(handle);
  oText = decrypt(mg->theKey, text, weak, crg, ag);
  // Fallback: if chosen strongness doesn't work, try the other one
  if (!oText)
    oText = decrypt(mg->theKey, text, !weak, crg, ag);
  MemHandleUnlock(handle);
  if (!oText) {
    FrmAlert(BadKeyError);
    return;
  }
  ReplaceText(fld, oText);
  MemSet(oText, StrLen(oText), 0);
  MemPtrFree(oText);
  CleanKey();
  CtlSetValue(GetObjectPtr(CopyCheck), false);
}


static void
CopyFieldToBuffer()
{
  FieldPtr fld = GetObjectPtr(TextField);
  MemHandle handle = FldGetTextHandle(fld);
  Char *text;
  Int16 len, mark;

  if (!handle) return; /* nothing to do */
  text = MemHandleLock(handle);
  len = StrLen(text);
  if (CtlGetValue(GetObjectPtr(CopyCheck))) {
    mark = len;
  } else {
    MemSet(text, len, 0); // clean data
    mark = 0;
  }
  MemHandleUnlock(handle);
  FldSetSelection(fld, 0, mark);
  FldCopy(fld);
}


static void
DoScroll(WinDirectionType dir, UInt16 amount)
{
  FieldPtr fld = GetObjectPtr(TextField);
  if (FldScrollable(fld, dir))
    FldScrollField(fld, amount, dir);
}


static void
FieldCommand(UInt16 cmd, FieldPtr fld)
{
  if (!fld)
    return;

  switch(cmd) {
    case EditUndo:
      FldUndo(fld);
      break;
    case EditCut:
      FldCut(fld);
      break;
    case EditCopy:
      FldCopy(fld);
      break;
    case EditPaste:
      FldPaste(fld);
      break;
    case EditSelectAll:
      FldSetSelection(fld, 0, FldGetTextLength(fld));
      break;
    case EditKeyboard:
      SysKeyboardDialogV10();
      break;
  }
}


static Boolean
MainFormHandleEvent(
        EventPtr eptr,
        cipher_globals_t *mg,
        crypt_globals_t *crg,
        armour_globals_t *ag)
{
  Boolean handled = false;
  FormPtr frm = FrmGetActiveForm();
  Int16 idx;
  Boolean quit_da = false;

  switch(eptr->eType) {
    case ctlSelectEvent:
      switch(eptr->data.ctlSelect.controlID) {
        case CipherButton:
          Cipher(false, mg, crg, ag);
          handled = true;
          break;

        case DecipherButton:
          Decipher(false, mg, crg, ag);
          handled = true;
          break;

        case UpButton:
          DoScroll(winUp, 1);
          handled = true;
          break;

        case DownButton:
          DoScroll(winDown, 1);
          handled = true;
          break;

        case QuitButton:
          CopyFieldToBuffer();
          quit_da = true;
          handled = true;
          break;
      }
      break;

    case keyDownEvent:
      if (ChrIsHardKey(eptr->data.keyDown.chr)) {
        EvtAddEventToQueue(eptr);
        quit_da = true;
        handled = true;
        break;
      }
      switch(eptr->data.keyDown.chr) {
        case pageUpChr:
          DoScroll(winUp, 9);
          handled = true;
          break;

        case pageDownChr:
          DoScroll(winDown, 9);
          handled = true;
          break;
      }
      break;

    case menuEvent:
      switch(eptr->data.menu.itemID) {
        case EditUndo:
        case EditCut:
        case EditCopy:
        case EditPaste:
        case EditSelectAll:
        case EditKeyboard:
          idx = FrmGetFocus(frm);
          if (idx != -1)
            FieldCommand(eptr->data.menu.itemID, FrmGetObjectPtr(frm, idx));
          handled = true;
          break;

        case CipherEncrypt:
          Cipher(false, mg, crg, ag);
          handled = true;
          break;

        case CipherDecrypt:
          Decipher(false, mg, crg, ag);
          handled = true;
          break;

        case CipherWEncrypt:
          Cipher(true, mg, crg, ag);
          handled = true;
          break;

        case CipherWDecrypt:
          Decipher(true, mg, crg, ag);
          handled = true;
      }
      break;

    case frmOpenEvent:
      // The form init is now in StartApplication.
      handled = true;
      break;

    case appStopEvent:
      EvtAddEventToQueue(eptr);
      quit_da = true;
      handled = true;
      break;
	default:
  }

  if (!handled)
    FrmHandleEvent(frm, eptr);

  return quit_da;
}


static void
StartApplication()
{
  FormPtr form;
  FieldPtr fld = NULL;
  Int16 id;

  // Copy eventually selected text
  if ((form = FrmGetActiveForm())) {
    if ((id = FrmGetFocus(form)) != -1) {
      switch(FrmGetObjectType(form, id)) {
        case frmFieldObj:
          fld = FrmGetObjectPtr(form, id);
          break;

        case frmTableObj:
          fld = TblGetCurrentField((TableType *) fld);
		  break;
		default:
      }
    }

    if (fld)
      FldCopy(fld);
  }

  form = FrmInitForm(MainForm);
  FrmSetActiveForm(form);
  FldPaste(GetObjectPtr(TextField));
  FrmSetFocus(form, FrmGetObjectIndex(form, KeyField));
  FrmDrawForm(form);
}


static void
StopApplication()
{
  FrmReturnToForm(0);
}


static void
EventLoop()
{
  EventType e;
  UInt16 err;
  cipher_globals_t *mg = NULL;
  crypt_globals_t *crg = NULL;
  armour_globals_t *ag = NULL;

  if ((mg = InitCipherGlobals()) &&
      (crg = InitCryptGlobals()) &&
      (ag = InitArmourGlobals()))
  {
    do {
      EvtGetEvent(&e, evtWaitForever);
      if (SysHandleEvent(&e))
        continue;
      if (MenuHandleEvent(NULL, &e, &err))
        continue;
    } while (!MainFormHandleEvent(&e, mg, crg, ag));
  }

  FreeCryptGlobals(crg);
  FreeArmourGlobals(ag);
  FreeCipherGlobals(mg);
}

// EOF
