/*
 * File:    crypt.h
 * Author:  Holger Klawitter
 */

// IDEA Specific Constants
#define INTERNAL_KEY_LENGTH 52
#define ROUNDS 8
#define BLOCK_SIZE 8
#define KEY_LENGTH 16

typedef struct {
  UInt16* ks;
} crypt_globals_t;

static Char *encrypt(Char* key, Char* text, Boolean weakEncrypt, crypt_globals_t *crg, armour_globals_t *ag);

/*
 * Decrypts an ASCII armoured text and returns the result.
 * key   - The key
 * aText - An encrypted ASCII armoured text.
 * weakEncrypt - uses old, weaker encryption scheme
 *
 * returns the decrypted text or NULL if the unarmouring or the
 *  decryption could identify the text as apparently not an
 *  encrypted or not matching to the key.
 */
static Char *decrypt(Char* key, Char* aText, Boolean weakEncrypt, crypt_globals_t *crg, armour_globals_t *ag);

static UInt16 mul(UInt32 a, UInt32 b);
static UInt16 inv(Int32 xx);
static crypt_globals_t *InitCryptGlobals(void);
static void FreeCryptGlobals(crypt_globals_t *crg);

// EOF
